@@ -69,6 +69,12 @@ describe "MySQL", '#create_table' do
     @db.set_column_type :dolls, :a, Integer, :auto_increment=>true
     @db.schema(:dolls).first.last[:auto_increment].must_equal true
   end
+
+  it "should create generated column" do
+    skip("generated columns not supported, skipping test") unless @db.supports_generated_columns?
+    @db.create_table(:dolls){String :a; String :b, generated_always_as: Sequel.function(:CONCAT, :a, 'plus')}
+    @db.schema(:dolls)[1][1][:generated].must_equal true
+  end
 end
 
 if [:mysql, :mysql2].include?(DB.adapter_scheme)
@@ -185,7 +191,6 @@ describe "A MySQL dataset" do
   it "should have explain output" do
     @d.explain.must_be_kind_of(String)
     @d.explain(:extended=>true).must_be_kind_of(String)
-    @d.explain.wont_equal @d.explain(:extended=>true)
   end
 
   it "should correctly literalize strings with comment backslashes in them" do
@@ -204,6 +209,13 @@ describe "A MySQL dataset" do
     ps.call(:v => 1, :n => 'b')
     ds.all.must_equal [{:value=>1, :name=>'b'}]
   end
+
+  it "should support generated columns" do
+    skip("generated columns not supported, skipping test") unless DB.supports_generated_columns?
+    DB.alter_table(:items) {add_column :b, String, :generated_always_as => Sequel.function(:CONCAT, :name, 'plus')}
+    @d.insert(name: 'hello')
+    @d.first[:b].must_equal 'helloplus'
+  end
 end
 
 describe "Dataset#distinct" do
@@ -220,6 +232,8 @@ describe "Dataset#distinct" do
   end
 
   it "#distinct with arguments should return results distinct on those arguments" do
+    skip("ONLY_FULL_GROUP_BY sql_mode set, skipping DISTINCT ON emulation test") if @db.get(Sequel.lit '@@sql_mode').include?('ONLY_FULL_GROUP_BY')
+
     @ds.insert(20, 10)
     @ds.insert(30, 10)
     @ds.order(:b, :a).distinct.map(:a).must_equal [20, 30]
@@ -274,7 +288,7 @@ describe "A MySQL database" do
     @db = DB
     @db.create_table! :test2 do
       text :name
-      integer :value
+      Integer :value
     end
   end
   after(:all) do
@@ -502,6 +516,15 @@ describe "A MySQL database" do
     @db << 'DELETE FROM items'
     @db[:items].first.must_be_nil
   end
+
+  it "should have schema handle generated columns" do
+    skip("generated columns not supported, skipping test") unless @db.supports_generated_columns?
+    @db.create_table(:items) {String :a}
+    @db.alter_table(:items){add_column :b, String, :generated_always_as=>Sequel.function(:CONCAT, :a, 'plus'), :generated_type=>:stored, :unique=>true}
+    @db.schema(:items)[1][1][:generated].must_equal true
+    @db.alter_table(:items){add_column :c, String, :generated_always_as=>Sequel.function(:CONCAT, :a, 'minus'), :generated_type=>:virtual}
+    @db.schema(:items)[2][1][:generated].must_equal true
+  end
 end  
 
 # Socket tests should only be run if the MySQL server is on localhost
@@ -528,11 +551,6 @@ describe "A MySQL database" do
     db = Sequel.connect(DB.opts.merge(:read_timeout=>22342))
     db.test_connection
   end
-
-  it "should accept a connect_timeout option when connecting" do
-    db = Sequel.connect(DB.opts.merge(:connect_timeout=>22342))
-    db.test_connection
-  end
 end
 
 describe "MySQL foreign key support" do
