@@ -10,7 +10,7 @@ require_relative "../../lib/sequel/core"
 
 ENV['MT_NO_PLUGINS'] = '1' # Work around stupid autoloading of plugins
 gem 'minitest'
-require 'minitest/autorun'
+require 'minitest/global_expectations/autorun'
 require 'minitest/hooks/default'
 require 'minitest/shared_description'
 
