@@ -30,6 +30,7 @@ describe "Class Table Inheritance Plugin" do
     end 
     class ::Manager < Employee
       one_to_many :staff_members, :class=>:Staff
+      one_to_one :first_staff_member, :clone=>:staff_members, :order=>:id
     end 
     class ::Executive < Manager
     end 
@@ -140,7 +141,7 @@ describe "Class Table Inheritance Plugin" do
   end
   
   it "should update rows in all tables" do
-    Executive.first.update(:name=>'Ex2', :num_managers=>8, :num_staff=>9)
+    Executive[:id=>@i4].update(:name=>'Ex2', :num_managers=>8, :num_staff=>9)
     @db[:employees][:id=>@i4].must_equal(:id=>@i4, :name=>'Ex2', :kind=>'Executive')
     @db[:managers][:id=>@i4].must_equal(:id=>@i4, :num_staff=>9)
     @db[:executives][:id=>@i4].must_equal(:id=>@i4, :num_managers=>8)
@@ -150,6 +151,8 @@ describe "Class Table Inheritance Plugin" do
     m = Staff.first.manager
     m.must_equal Manager[@i4]
     m.must_be_kind_of(Executive)
+    Staff.first.update(:manager => Manager[@i3])
+    Staff.first.manager.must_equal Manager[@i3]
   end
   
   it "should handle eagerly loading many_to_one relationships" do
@@ -164,6 +167,18 @@ describe "Class Table Inheritance Plugin" do
   
   it "should handle one_to_many relationships" do
     Executive.first(:name=>'Ex').staff_members.must_equal [Staff[@i2]]
+    i6 = @db[:employees].insert(:name=>'S2', :kind=>'Staff')
+    @db[:staff].insert(:id=>i6, :manager_id=>@i4)
+    Executive.first(:name=>'Ex').add_staff_member(i6)
+    Executive.first(:name=>'Ex').staff_members{|ds| ds.order(:id)}.must_equal [Staff[@i2], Staff[i6]]
+  end
+  
+  it "should handle one_to_many relationships" do
+    Executive.first(:name=>'Ex').first_staff_member.must_equal Staff[@i2]
+    i6 = @db[:employees].insert(:name=>'S2', :kind=>'Staff')
+    @db[:staff].insert(:id=>i6, :manager_id=>@i4)
+    Executive.first(:name=>'Ex').first_staff_member = Staff[i6]
+    Executive.first(:name=>'Ex').staff_members.must_equal [Staff[i6]]
   end
   
   it "should handle eagerly loading one_to_many relationships" do
@@ -1709,7 +1724,9 @@ describe "Sequel::Plugins::Tree" do
 
     it "iterate top-level nodes in order" do
       @Node.roots_dataset.count.must_equal 5
-      @Node.roots.map{|p| p.name}.must_equal %w'one two three four five'
+      @Node.roots.map(&:name).must_equal %w'one two three four five'
+      @Node.where(:name=>%w'one two.one').roots_dataset.count.must_equal 1
+      @Node.where(:name=>%w'one two.one').roots.map(&:name).must_equal %w'one'
     end
   
     it "should have children" do
@@ -2026,8 +2043,9 @@ describe "Sequel::Plugins::ConstraintValidations" do
       max_length 6, :minlen, opts.merge(:name=>:maxl2)
       operator :<, 'm', :exactlen, opts.merge(:name=>:lt)
       operator :>=, 5, :num, opts.merge(:name=>:gte)
+      presence [:m1, :m2, :m3], opts.merge(:name=>:pm)
     end
-    @valid_row = {:pre=>'a', :exactlen=>'12345', :minlen=>'12345', :maxlen=>'12345', :lenrange=>'1234', :lik=>'fooabc', :ilik=>'FooABC', :inc=>'abc', :uniq=>'u', :num=>5}
+    @valid_row = {:pre=>'a', :exactlen=>'12345', :minlen=>'12345', :maxlen=>'12345', :lenrange=>'1234', :lik=>'fooabc', :ilik=>'FooABC', :inc=>'abc', :uniq=>'u', :num=>5, :m1=>'a', :m2=>1, :m3=>'a'}
     @violations = [
       [:pre, [nil, '', ' ']],
       [:exactlen, [nil, '', '1234', '123456', 'n1234']],
@@ -2068,6 +2086,41 @@ describe "Sequel::Plugins::ConstraintValidations" do
         end
       end
 
+      try = @valid_row.dup
+      if @validation_opts[:allow_nil]
+        [:m1, :m2, :m3].each do |c|
+          @ds.insert(try.merge(c=>nil))
+          @ds.delete
+        end
+        @ds.insert(try.merge(:m1=>nil, :m2=>nil))
+        @ds.delete
+        @ds.insert(try.merge(:m1=>nil, :m3=>nil))
+        @ds.delete
+        @ds.insert(try.merge(:m2=>nil, :m3=>nil))
+        @ds.delete
+        @ds.insert(try.merge(:m1=>nil, :m2=>nil, :m3=>nil))
+        @ds.delete
+      else
+        [:m1, :m2, :m3].each do |c|
+          proc{@ds.insert(try.merge(c=>nil))}.must_raise(Sequel::DatabaseError)
+        end
+        proc{@ds.insert(try.merge(:m1=>nil, :m2=>nil))}.must_raise(Sequel::DatabaseError)
+        proc{@ds.insert(try.merge(:m1=>nil, :m3=>nil))}.must_raise(Sequel::DatabaseError)
+        proc{@ds.insert(try.merge(:m2=>nil, :m3=>nil))}.must_raise(Sequel::DatabaseError)
+        proc{@ds.insert(try.merge(:m1=>nil, :m2=>nil, :m3=>nil))}.must_raise(Sequel::DatabaseError)
+      end
+
+      unless @db.database_type == :oracle
+        [:m1,  :m3].each do |c|
+          proc{@ds.insert(try.merge(c=>''))}.must_raise(Sequel::DatabaseError)
+        end
+        proc{@ds.insert(try.merge(:m1=>'', :m3=>''))}.must_raise(Sequel::DatabaseError)
+        proc{@ds.insert(try.merge(:m1=>'', :m2=>nil))}.must_raise(Sequel::DatabaseError)
+        proc{@ds.insert(try.merge(:m1=>nil, :m3=>''))}.must_raise(Sequel::DatabaseError)
+        proc{@ds.insert(try.merge(:m2=>nil, :m3=>''))}.must_raise(Sequel::DatabaseError)
+        proc{@ds.insert(try.merge(:m1=>'', :m2=>nil, :m3=>''))}.must_raise(Sequel::DatabaseError)
+      end
+
       # Test for dropping of constraint
       @db.alter_table(:cv_test){validate{drop :maxl2}}
       @ds.insert(@valid_row.merge(:minlen=>'1234567'))
@@ -2092,6 +2145,34 @@ describe "Sequel::Plugins::ConstraintValidations" do
           c.new(try).wont_be :valid?
         end
       end
+
+      try = @valid_row.dup
+      if @validation_opts[:allow_nil]
+        [:m1, :m2, :m3].each do |col|
+          c.new(try.merge(col=>nil)).must_be :valid?
+        end
+        c.new(try.merge(:m1=>nil, :m2=>nil)).must_be :valid?
+        c.new(try.merge(:m1=>nil, :m3=>nil)).must_be :valid?
+        c.new(try.merge(:m2=>nil, :m3=>nil)).must_be :valid?
+        c.new(try.merge(:m1=>nil, :m2=>nil, :m3=>nil)).must_be :valid?
+      else
+        [:m1, :m2, :m3].each do |col|
+          c.new(try.merge(col=>nil)).wont_be :valid?
+        end
+        c.new(try.merge(:m1=>nil, :m2=>nil)).wont_be :valid?
+        c.new(try.merge(:m1=>nil, :m3=>nil)).wont_be :valid?
+        c.new(try.merge(:m2=>nil, :m3=>nil)).wont_be :valid?
+        c.new(try.merge(:m1=>nil, :m2=>nil, :m3=>nil)).wont_be :valid?
+      end
+      c.new(try.merge(:m1=>'', :m2=>nil)).wont_be :valid?
+      c.new(try.merge(:m1=>nil, :m3=>'')).wont_be :valid?
+      c.new(try.merge(:m2=>nil, :m3=>'')).wont_be :valid?
+      c.new(try.merge(:m1=>'', :m2=>nil, :m3=>'')).wont_be :valid?
+      [:m1,  :m3].each do |col|
+        c.new(try.merge(col=>'')).wont_be :valid?
+      end
+      c.new(try.merge(:m1=>'', :m3=>'')).wont_be :valid?
+
       c.db.constraint_validations = nil
     end
   end
@@ -2116,6 +2197,9 @@ describe "Sequel::Plugins::ConstraintValidations" do
           String :inc
           String :uniq, :null=>false
           Integer :num
+          String :m1
+          Integer :m2
+          String :m3
           validate(&validate_block)
         end
       end
@@ -2162,6 +2246,9 @@ describe "Sequel::Plugins::ConstraintValidations" do
             add_column :form, String
           end
           add_column :num, Integer
+          add_column :m1, String
+          add_column :m2, Integer
+          add_column :m3, String
           validate(&validate_block)
         end
       end
@@ -2307,3 +2394,30 @@ describe "string_agg extension" do
     @ds.select_group(:id).select_append(Sequel.string_agg(:s).order(:s).distinct.as(:v)).map([:id, :v]).must_equal [[1, 'a,b,c'], [2, 'aa,bb']]
   end
 end
+
+describe "insert_conflict plugin" do
+  before(:all) do
+    @db = DB
+    @db.create_table!(:ic_test) do
+      primary_key :id
+      String :s, :unique=>true
+      Integer :o
+    end
+    @model = Class.new(Sequel::Model)
+    @model.set_dataset @db[:ic_test]
+    @model.plugin :insert_conflict
+  end
+  after(:all) do
+    @db.drop_table?(:ic_test)
+  end
+
+  it "should allow Model#insert_conflict to work" do
+    ic_opts = {:target=>:s, :update => {:o => Sequel[:excluded][:o]}}
+    @model.new(:s=>'A', :o=>1).insert_conflict(ic_opts).save
+    @model.select_order_map([:s, :o]).must_equal [['A', 1]]
+    @model.new(:s=>'A', :o=>2).insert_conflict(ic_opts).save
+    @model.select_order_map([:s, :o]).must_equal [['A', 2]]
+    @model.new(:s=>'B', :o=>3).insert_conflict(ic_opts).save
+    @model.select_order_map([:s, :o]).must_equal [['A', 2], ['B', 3]]
+  end
+end if (DB.database_type == :postgres && DB.server_version >= 90500) || (DB.database_type == :sqlite && DB.sqlite_version >= 32400)
